#
# See the file "LICENSE" for the full license governing this code.
#
#
# This function checks if the specfied patches are installed.
# For type = db (default) the function will return a hash with all homes and their patch details
#   that needs at least one patch installed.
# For type = grid the function will return a hash with only patches that haven't been installed yet.
#   There is a difference because of the way patches are applied for grid, opatchauto vs regular provider.
#
# It will check all registered Oracle homes for the installed patches.
#
Puppet::Functions.create_function(:'ora_install::ora_patches_missing') do
  dispatch :check_patch_list do
    param 'Hash', :patch_list
    optional_param 'String', :type
    return_type 'Hash'
  end

  def check_patch_list(patch_list, type='db')
    converted_patch_list = patch_list.map do |patch, patch_info|
      if patch_info['sub_patches']
        patch_info['ensure'].to_s == 'absent' ? patch_info['sub_patches'].map { |x| "#{patch.split(':')[0]}:#{x}:remove" } : patch_info['sub_patches'].map { |x| "#{patch.split(':')[0]}:#{x}" }
      else
        patch_info['ensure'].to_s == 'absent' ? "#{patch}:remove" : patch
      end
    end.flatten
    log("converted patch list: #{converted_patch_list}")
    scope = closure_scope
    oracle_homes = scope['ora_install_homes'] || []
    if oracle_homes.empty?
      return patch_list
    else
      oracle_homes.each do |key, value|
        if value['opatch_version'] == 'Invalid'
          fail "Either ORACLE_HOME '#{key}' or '#{key}/OPatch/opatch' not found. Faulty oraInventory entry? Please fix!"
        end
      end
      existing_patches = oracle_homes.nil? ? [] : oracle_homes.collect { |key, val| val['installed_patches'].map { |patch| "#{key}:#{patch}" } }.flatten
      log("existing patches: #{existing_patches}")
      missing_patches = converted_patch_list.select do | patch |
        if patch =~ /remove$/
          if existing_patches.include?(patch.chomp(':remove'))
            true
          else
            false
          end
        else
          !existing_patches.include?(patch)
        end
      end
      log("missing patches: #{missing_patches}")
      if missing_patches.empty?
        return {}
      else
        homes_to_be_patched = missing_patches.map {|patch| patch.split(':')[0]}.uniq
        if type == 'grid'
          missing_patch_numbers = missing_patches.collect { |element| element.split(":")[1] }
          patch_list_out = patch_list.select { |home, _patch| (missing_patch_numbers & patch_list[home]['sub_patches']).any? }
        elsif type == 'db'
          patch_list_out = patch_list.select { |home, _patch| homes_to_be_patched.include?(home.split(':')[0]) }
        else
          fail "Wrong type (#{type}) specified, only 'grid' and 'db' are supported"
        end
        log("patch_list_out: #{patch_list_out}")
        return patch_list_out
      end
    end
  end

  def log(msg)
    Puppet::Util::Log.create(
      :level   => :debug,
      :message => msg,
      :source  => 'ora_patches_missing'
    )
  end
end
