#
# See the file "LICENSE" for the full license governing this code.
#
require 'puppet_x/enterprisemodules/ora_install/core'

module Puppet
  #
  Type.newtype(:ora_opatch) do
    include EasyType
    include EasyType::Helpers
    include PuppetX::EnterpriseModules::OraInstall::Commands
    include PuppetX::EnterpriseModules::OraInstall::GetHomes

    desc 'This resource allows you to manage opatch patches on a specific database home.'

    set_command([:opatch, :opatchauto])

    to_get_raw_resources do
      provider(:base).installed_patches
    end

    validate do
      fail "sub_patches may not be same as main patch id." if sub_patches && sub_patches.include?(patch_id)
    end

    on_create do
      return if refreshonly == :true
      provider.remove_incompatible_patches
      provider.apply_patch(get_patches)
      remove_unzipped_file(source)
    end

    on_modify do
      raise 'Internal error. A patch is either there ot not. It cannot be modified.'
    end

    on_destroy do
      #
      # Only the opatchauto requires the source to be extracted
      # The other providers use stored information.
      #
      return if refreshonly == :true
      if provider.name == :opatchauto
        get_patches
        remove_unzipped_file(source)
      else
        provider.remove_patch('')
      end
    end

    def refresh
      return if refreshonly == :false
      installed = patch_installed?("#{oracle_product_home_dir}:#{patch_id}")
      return if installed && self.ensure == :present
      return if !installed && self.ensure == :absent
      if ( self.ensure == :present and !installed )
        provider.remove_incompatible_patches
        provider.apply_patch(get_patches)
        remove_unzipped_file(source)
      else
        if provider.name == :opatchauto
          provider.remove_incompatible_patches
          get_patches
          remove_unzipped_file(source)
        else
          provider.remove_patch('')
        end
      end
    end

    map_titles_to_attributes([
      /^((.*):(.*))$/, [:name, :oracle_product_home_dir, :patch_id]
    ])

    parameter :name
    parameter :patch_id
    parameter :oracle_product_home_dir
    parameter :orainst_dir
    parameter :ocmrf_file
    parameter :source
    parameter :tmp_dir
    parameter :source_dir
    parameter :sub_patches
    parameter :remove_patches_before
    parameter :force_remove
    parameter :refreshonly
    property  :comment
    property  :ensure


    def generate
      #
      # Add new resources to the catalog based on the sub_patches
      #
      if sub_patches && sub_patches.any?
        if puppet_version_lower_then('4.4.0')
          #
          # Below version 4.4.0, Puppet doesn't recognise the relations
          # between the major resource and resources generated by `generate`.
          #
          # This means all generated resources will be positioned at the end.
          # For ora_opatches, this is **not** good. We need to manage the ordering
          # of patches. Therefor we skip this function when Puppet version is to low.
          #
          raise 'Using sub_patches needs Puppet 4.4.0 or higher.'
        end
        resources = sub_patches.collect do |sub_patch|
          hash = to_hash
          hash[:name] = "#{oracle_product_home_dir}:#{sub_patch}"
          hash[:patch_id] = sub_patch
          hash[:source_dir] = "#{hash[:source_dir]}/#{sub_patch}"

          hash.delete(:sub_patches)
          self.class.new(hash)
        end
        self[:ensure] = :absent
        resources
      end
    end

    def patch_installed?(patch)
      installed_patches.any? {|p| p[:name] == patch}
    end

    def get_patches
      parameters[:source].process
      FileUtils.chmod_R(0o777, source) # So oracle and grid users can access them
      check_source_dir(source)
    end

    def check_source_dir(parent)
      patch_source_dir = Dir.glob("#{parent}/**/#{source_dir}").first
      raise "#{source} doesn't contain folder #{source_dir}" unless patch_source_dir
      patch_source_dir
    end

    def remove_unzipped_file(source)
      Puppet::Util::Execution.execute("rm -rf #{tmp_dir}")
    end

    def puppet_version_lower_then(version)
      current_version = Gem::Version.new(Puppet.version)
      requested_version = Gem::Version.new(version)
      current_version <= requested_version
    end
  end
end
