#
# ora_install::opatch
#
# @summary [DEPRECATED] Wrapper defined type for installing patches.
# Please use[`ora_opatch`](/docs/ora_install/ora_opatch.html) to manage patches of your database software.
# 
# Here is an example on how to use this:
# 
# ```puppet
# ora_install::opatch{'19121551_db_patch':
#   ensure                    => 'present',
#   oracle_product_home       => hiera('oracle_home_dir'),
#   patch_id                  => '19121551',
#   patch_file                => 'p19121551_112040_Linux-x86-64.zip',
#   user                      => hiera('oracle_os_user'),
#   group                     => 'oinstall',
#   download_dir              => hiera('oracle_download_dir'),
#   ocmrf                     => true,
#   require                   => Ora_install::Opatchupgrade['112000_opatch_upgrade_db'],
#   puppet_download_mnt_point => hiera('oracle_source'),
# }
# ```
#
# @param [Optional[Integer]] bundle_sub_folder
#    The folder in the zip file used for the specified patch.
#
# @param [Optional[Integer]] bundle_sub_patch_id
#    The subid of the patch.
#
# @param [Boolean] clusterware
#    When true use opatch auto.
#    Default value `false`
#
# @param [Stdlib::Absolutepath] download_dir
#    The directory where the Puppet software puts all downloaded files.
#    Before Puppet can actually use remote files, they must be downloaded first. Puppet uses this directory to put all files in.
#    The default value is: `/install`
#
# @param [Enum[present, absent]] ensure
#    State to obtain.
#    The ensure attribute can be one of two values:
#    - present
#    - absent
#    When you specify `present`, Puppet will make sure the resource is available with all specified options and properties.
#    When the resource is already available(installed), and all attributes are as the are specified, Puppet will do nothing.
#    When you specify `absent`, Puppet will remove the resource if it is available. If it is not installed, Puppet will do nothing.
#
# @param [String[1]] group
#    The os group to use for these Oracle puppet definitions.
#    The default value is: `dba`
#
# @param [Boolean] ocmrf
#    Wether or not to use OCMRF file.
#    The default value is: `false`
#
# @param [Stdlib::Absolutepath] oracle_product_home
#    The Oracle home to use.
#
# @param [String[1]] patch_file
#    The name of the patch file.
#
# @param [Integer] patch_id
#    The ID of the patch to apply.
#
# @param [Optional[String[1]]] puppet_download_mnt_point
#    The base path of all remote files for the defined type or class.
#    The default value is: `puppet:///modules/ora_install`
#
# @param [Optional[Boolean]] remote_file
#    **DEPRECATED**
#    This parameter is not being used anymore and will be removed in a future release.
#    `true`.
#
# @param [String[1]] user
#    The user used for the specified installation.
#    The install class will not create the user for you. You must do that yourself.
#    The default value is: `oracle`
#
#
# See the file "LICENSE" for the full license governing this code.
#
define ora_install::opatch(
  Enum[present, absent] $ensure                    = present,
  Stdlib::Absolutepath  $oracle_product_home       = undef,
  Integer               $patch_id                  = undef,
  String[1]             $patch_file                = undef,
  Boolean               $clusterware               = false, # opatch auto or opatch apply
  Optional[Integer]     $bundle_sub_patch_id       = undef,
  Optional[Integer]     $bundle_sub_folder         = undef,
  String[1]             $user                      = 'oracle',
  String[1]             $group                     = 'dba',
  Stdlib::Absolutepath  $download_dir              = '/install',
  Boolean               $ocmrf                     = false,
  Optional[String[1]]   $puppet_download_mnt_point = undef,
  Optional[Boolean]     $remote_file               = undef,
  Boolean               $allow_insecure            = lookup({name => 'allow_insecure', default_value => false}),
)
{
  ora_install { $title: ensure => 'present'}

  if ( $remote_file in [true,false] ) {
    deprecation (remote_file, "Parameter 'remote_file' is deprecated and will be removed in a future release")
  }

  $exec_path = '/usr/local/bin:/bin:/usr/bin:/usr/local/sbin:/usr/sbin:/sbin:'

  case $::kernel {
    'Linux': {
      $ora_inst_path = '/etc'
    }
    'SunOS': {
      $ora_inst_path = '/var/opt/oracle'
    }
    default: {
        fail("Unrecognized operating system ${::kernel}, please use it on a Linux host")
    }
  }

  if ( $puppet_download_mnt_point ) {
    $mount_point =  $puppet_download_mnt_point
  } else {
    $mount_point = 'puppet:///modules/ora_install/'
  }

  easy_type::debug_evaluation() # Show local variable on extended debug

  case $::kernel {
    'Linux', 'SunOS': {
      if $ensure == 'present' {
        archive { "${download_dir}/${patch_file}":
          ensure         => present,
          allow_insecure => $allow_insecure,
          user           => $user,
          group          => $group,
          extract        => true,
          extract_path   => $download_dir,
          source         => "${mount_point}/${patch_file}",
          creates        => "${download_dir}/${patch_id}",
          cleanup        => false,
        }
      }

      # sometimes the bundle patch inside an other folder
      if ( $bundle_sub_folder ) {
        $extracted_patch_dir = "${download_dir}/${patch_id}/${bundle_sub_folder}"
      } else {
        $extracted_patch_dir = "${download_dir}/${patch_id}"
      }

      if ( $ocmrf ) {

        db_opatch{ "${patch_id} ${title}":
          ensure                  => $ensure,
          patch_id                => $patch_id,
          os_user                 => $user,
          oracle_product_home_dir => $oracle_product_home,
          orainst_dir             => $ora_inst_path,
          extracted_patch_dir     => $extracted_patch_dir,
          ocmrf_file              => "${oracle_product_home}/OPatch/ocm.rsp",
          bundle_sub_patch_id     => $bundle_sub_patch_id,
          opatch_auto             => $clusterware,
        }

      } else {

        db_opatch{ "${patch_id} ${title}":
          ensure                  => $ensure,
          patch_id                => $patch_id,
          os_user                 => $user,
          oracle_product_home_dir => $oracle_product_home,
          orainst_dir             => $ora_inst_path,
          extracted_patch_dir     => $extracted_patch_dir,
          bundle_sub_patch_id     => $bundle_sub_patch_id,
          opatch_auto             => $clusterware,
        }

      }

    }
    default: {
      fail('Unrecognized operating system')
    }
  }
}
